from pyramid.view import view_config

from pyramid.httpexceptions import HTTPBadRequest
from ..models.c2cgeoform_demo import Address


@view_config(route_name='addresses', request_method='GET', renderer='json')
def addresses(request):
    if 'term' not in request.params:
        return HTTPBadRequest()
    term = '%%%s%%' % request.params['term']
    query = request.dbsession.query(Address).filter(Address.label.ilike(term))
    return [{'id': addr.id, 'label': addr.label} for addr in query]



# pylint: disable=no-self-use

import pprint


class AbstractViewsTests():
    """
    Base class for views testing
    """
    _prefix = None  # url prefix (index view url). Example : /users

    def get(self, test_app, path='', locale='en', status=200, **kwargs):
        return test_app.get(
            '{}{}'.format(self._prefix, path),
            headers={
                'Cookie': '_LOCALE_={}'.format(locale)
            },
            status=status,
            **kwargs)

    def get_item(self, test_app, item_id, **kwargs):
        return self.get(test_app, '/{}'.format(item_id), **kwargs)

    def check_left_menu(self, resp, title):
        link = resp.html.select_one('.navbar li.active a')
        assert 'http://localhost{}'.format(self._prefix) == link.attrs['href']
        assert title == link.getText()

    def check_grid_headers(self, resp, expected_col_headers, check_actions=True):
        pp = pprint.PrettyPrinter(indent=4)
        effective_cols = [(th.attrs['data-field'], th.getText(), th.attrs['data-sortable'])
                          for th in resp.html.select('th')]
        expected_col_headers = [(x[0], x[1], len(x) == 3 and x[2] or 'true') for x in expected_col_headers]
        assert expected_col_headers == effective_cols, \
            str.format('\n\n{}\n\n differs from \n\n{}',
                       pp.pformat(expected_col_headers),
                       pp.pformat(effective_cols))
        if check_actions:
            actions = resp.html.select_one('th[data-field="actions"]')
            assert 'false' == actions.attrs['data-sortable']

    def check_new_button(self, resp):
        assert 1 == len(list(filter(lambda x: str(x.contents) == "['New']",
                                    resp.html.findAll('a'))))

    def check_search(self, test_app, search='', offset=0, limit=10, sort='', order='', total=None, **kwargs):
        json = test_app.get(
            '{}/grid.json'.format(self._prefix),
            params=dict(
                offset=offset,
                limit=limit,
                search=search,
                sort=sort,
                order=order,
                **kwargs
            ),
            status=200
        ).json
        if total is not None:
            assert total == json['total']
        return json

    def check_checkboxes(self, form, name, expected):
        for i, exp in enumerate(expected):
            field = form.get(name, index=i)
            checkbox = form.html.select_one('#{}'.format(field.id))
            label = form.html.select_one('label[for={}]'.format(field.id))
            assert exp['label'] == list(label.stripped_strings)[0]
            assert exp['value'] == checkbox['value']
            assert exp['checked'] == field.checked

    def get_first_field_named(self, form, name):
        return form.fields.get(name)[0]

    def set_first_field_named(self, form, name, value):
        form.fields.get(name)[0].value__set(value)

    def _check_sequence(self, sequence, expected):
        seq_items = sequence.select('.deform-seq-item')
        assert len(expected) == len(seq_items)
        for seq_item, exp in zip(seq_items, expected):
            self._check_mapping(seq_item, exp)

    def _check_mapping(self, mapping_item, expected):
        for exp in expected:
            input_tag = mapping_item.select_one('[name="{}"]'.format(exp['name']))
            if 'value' in exp:
                if exp.get('readonly', False):
                    item = mapping_item.select_one('.item-{}'.format(exp['name']))
                    assert input_tag is None
                    assert exp['value'] or '' == item.select('p').stripped_strings[0]
                elif input_tag.name == 'select':
                    self._check_select(input_tag, exp['value'])
                elif input_tag.name == 'textarea':
                    assert (exp['value'] or '') == (input_tag.string or '')
                else:
                    assert (exp['value'] or '') == input_tag.attrs.get('value', '')
            if exp.get('hidden', False):
                assert 'hidden' == input_tag['type']
            if 'label' in exp:
                label_tag = mapping_item.select_one('label[for="{}"]'.format(input_tag['id']))
                assert exp['label'] == label_tag.getText().strip()

    def _check_select(self, select, expected):
        for exp, option in zip(expected, select.find_all('option')):
            if 'text' in exp:
                assert exp['text'] == option.text
            if 'value' in exp:
                assert exp['value'] == option['value']
            if 'selected' in exp:
                assert exp['selected'] == ('selected' in option.attrs)

    def _check_submission_problem(self, resp, expected_msg):
        assert 'There was a problem with your submission' == \
            resp.html.select_one('div[class="error-msg-lbl"]').text
        assert 'Errors have been highlighted below' == \
            resp.html.select_one('div[class="error-msg-detail"]').text
        assert expected_msg == \
            resp.html.select_one("[class~='has-error']").select_one("[class~='help-block']").getText().strip()



from pkg_resources import resource_filename
from pyramid.i18n import get_localizer
from pyramid.threadlocal import get_current_request
from pyramid.config import Configurator
from deform import Form, widget
from translationstring import TranslationStringFactory


_ = TranslationStringFactory('c2cgeoform')


""" Default search paths for the form templates.
"""
default_search_paths = (
    resource_filename('c2cgeoform', 'templates/widgets'),
    resource_filename('deform', 'templates'))


default_map_settings = {
    'srid': 3857,
    'baseLayers': [{'type_': "OSM"}],
    'view': {
        'projection': 'EPSG:3857',
        'center': [829170, 5933942],
        'zoom': 7,
    },
    'fitSource': False,
    'fitMaxZoom': 14,
    'focusOnly': False,
    'geolocationTooltip': _('Zoom to current location'),
    'zoomInTipLabel': _('Zoom in'),
    'zoomOutTipLabel': _('Zoom out'),
    'mobile': None
}


def main(global_config, **settings):
    """
    This function returns a Pyramid WSGI application.
    In our case, this is only used for tests.
    """
    config = Configurator(settings=settings)
    return config.make_wsgi_app()


def includeme(config):
    """
    Function called when "c2cgeoform" is included in a project (with
    ``config.include('c2cgeoform')``).

    This function creates routes and views for c2cgeoform pages.
    """
    config.include('pyramid_chameleon')
    config.include('pyramid_beaker')  # use Beaker for session storage
    config.include('.routes')
    config.include('.views')
    config.add_static_view('c2cgeoform_static', 'static', cache_max_age=3600)
    config.add_static_view('deform_static', 'deform:static')
    config.add_translation_dirs('colander:locale', 'deform:locale', 'locale')

    init_deform(config.root_package.__name__)

    config.scan('c2cgeoform.views')


def translator(term):
    request = get_current_request()
    if request is None:
        return term
    else:
        return get_localizer(request).translate(term)


def init_deform(root_package):
    Form.set_zpt_renderer(default_search_paths, translator=translator)

    node_modules_root = '{}:node_modules'.format(root_package)

    registry = widget.default_resource_registry
    registry.set_js_resources(
        'typeahead', '0.10.5',
        '{}/typeahead.js/dist/typeahead.bundle.min.js'.
        format(node_modules_root))
    registry.set_css_resources(
        'typeahead', '0.10.5',
        'c2cgeoform:static/js/typeaheadjs.css')

    widget.MappingWidget.fields_template = 'mapping_fields'
    widget.FormWidget.fields_template = 'mapping_fields'
